from car_util import * # yeahhh don't do this
from roof import roof_rear_pts

####################
# back box behind roof
####################

def backbox_pts(t, airbox_pts, cfloor_pts, offs, bump_rear_ht):
    aback, atop = airbox_pts
    airbox_back = atop[-1][0]
    airbox_front = atop[0][-1]
    airbox_front2 = atop[1][-1]
    roof = roof_rear_pts(t, airbox_back, airbox_front, offs,
                         False, back_fwd[1], # bring rear back slightly
                         0.05, # bring rear line down slightly
                         0.12, # curve roof line
                         0.02, # bring width in slightly
                         xangle=-19.0) # angle slightly (20 is too much)

    # base line behind central floor
    boxbase = []
    for i in range(len(cfloor_pts)):
        pos = copy(cfloor_pts[i][0])
        pos[1] += back_fwd[1] + cockpit_floor_slope
        pos[2] = bump_rear_ht
        # move ends up a bit and forward a bit
        if i == 0 or i == 3:
            pos[1] -= 0.04
            pos[2] += 0.06
        # adjust stance to match floor which moved inwards a bit
        if i == 1:
            pos[0] -= 0.04
        if i == 2:
            pos[0] += 0.04
        boxbase.append(pos)
        t.add_joint(pos)

    # middle height back box line
    midbox = []
    for j, i in enumerate([7, 4, 3, 0]):
        pos = copy(roof[i])
        pos[1] -= 0.03
        # height matching inner airbox on inside, drops visually on outside
        if j in [2,1]:
            pos[2] = h1-0.04
        else:
            pos[2] = h1-0.04
        if False: # XXX no, this makes the diagonal uneven
            # expand out upper airbox width
            if j == 1:
                pos[0] -= 0.06
            if j == 2:
                pos[0] += 0.06
        midbox.append(pos)
        t.add_joint(pos)

    # middle height back box forward line
    midfront = []
    pos = copy(aback[0][0])
    pos[0] = -wpoints[-1]
    midfront.append(pos)
    midfront.append(aback[0][0])
    midfront.append(aback[1][0])
    pos = copy(aback[1][0])
    pos[0] = wpoints[-1]
    midfront.append(pos)
    for i, pos in enumerate(midfront):
        if i == 1 or i == 2:
            continue # skip points already in air box bottom
        t.add_joint(pos)

    return [roof, boxbase, midbox, midfront]


class BackBox(Group):

    def __init__(self):
        Group.__init__(self)
        self.dflt_pipe_diam = dthin

    def make(self, roof_pts, airbox_pts, cfloor_pts, offs, bump_rear_ht):
        bbox = backbox_pts(self, airbox_pts, cfloor_pts, offs, bump_rear_ht)
        [roof, boxbase, midbox, midfront] = bbox
        rbase, _, _, _ = roof_pts

        # add horizontal forward mid pipes
        for i in range(len(midbox)):
            self.add_pipe(midbox[i], midfront[i], dthick)

        # horizontal-width
        for i in range(len(midbox)-1):
            offs1, offs2 = AUTO, AUTO
            if i == 0:
                offs1 = jrrot
            if i == 2:
                offs2 = jlrot
            self.add_pipe(midbox[i], midbox[i+1], dthin, offs1, offs2)
        # vertical, floor to mid (doorframe)
        self.add_pipe(midfront[0], cfloor_pts[0][0], dthick)
        self.add_pipe(midfront[-1], cfloor_pts[-1][0], dthick)
        # diagonal
        jdrr = rotate_z(jdr, -15) # move these out way of doorsill
        jdlr = rotate_z(jdl, 15)  # this one too
        self.add_pipe(midfront[0], cfloor_pts[1][0], dthin, jdrr)
        self.add_pipe(midfront[-1], cfloor_pts[-2][0], dthin, jdlr)

        # diagonal-cross
        for i in range(len(midfront)-1):
            if i == 0:
                self.add_pipe(midfront[i+1], midbox[i], dthin, jlbrot3)
            else:
                self.add_pipe(midfront[i], midbox[i+1], dthin, jrbrot3)

        # add horizontal forward base pipes
        for i in range(len(boxbase)):
            self.add_pipe(boxbase[i], cfloor_pts[i][0], dthick)
        # down-diagonal forwards
        for i in range(len(boxbase)):
            self.add_pipe(midbox[i], cfloor_pts[i][0], dthin, jfdrot)
        # horizontal-width
        for i in range(len(boxbase)-1):
            self.add_pipe(boxbase[i], boxbase[i+1], dthick)
        # horizontal diagonals
        for i in range(len(boxbase)-1):
            offs1, offs2 = AUTO, AUTO
            if i == 1:
                offs1, offs2 = jlfrot3, jrbrot3
            if i != 0:
                self.add_pipe(boxbase[i+1], cfloor_pts[i][0],
                              dthin, offs1, offs2)
            else:
                self.add_pipe(boxbase[i], cfloor_pts[i+1][0])

        # add vertical back pipes
        for i in range(len(midbox)):
            self.add_pipe(midbox[i], boxbase[i], dthick)
        # vertical diagonals
        for i in range(len(boxbase)-1):
            if i != 0:
                self.add_pipe(boxbase[i], midbox[i+1])
                self.add_pipe(boxbase[i+1], midfront[i])
            else:
                self.add_pipe(boxbase[i+1], midbox[i])
                self.add_pipe(boxbase[i], midfront[i+1])

        ###
        # now the roofline
        ###

        # horizontal-forward
        for i in range(len(rbase)):
            self.add_pipe(roof[i], rbase[i])
        # inter-roof
        for i in range(len(rbase)-1):
            offs2 = AUTO
            if i in [0, 4]:
                offs2 = jrbd3
            if i in [2, 6]:
                offs2 = jlbd3
            if i in [1,2, 6]:
                self.add_pipe(roof[i+1], rbase[i], dthin, AUTO, offs2)
            else:
                self.add_pipe(roof[i], rbase[i+1], dthin, AUTO, offs2)
            self.add_pipe(roof[i], roof[i+1])

        # rear roof uprights
        ridxs = [7, 4, 3, 0]
        for i, i1 in enumerate(ridxs):
            self.add_pipe(roof[i1], midbox[i])
            #self.add_pipe(roof[i1], midfront[i])
            self.add_pipe(rbase[i1], midbox[i], dthick)
            # diagonals
            if i == 0:
                self.add_pipe(roof[4], midbox[i])
                self.add_pipe(rbase[4], midfront[i])
                self.add_pipe(rbase[i1], midbox[i+1], dthin, jbd)
            if i == 3:
                self.add_pipe(roof[3], midbox[i])
                self.add_pipe(rbase[3], midfront[i])
                self.add_pipe(rbase[i1], midbox[i-1], dthin, jbd)
        # link diagonally between two rooflines
        self.add_pipe(rbase[ridxs[0]], roof[ridxs[1]], dthin,
                      jrbrot3, jlfrot3)
        self.add_pipe(rbase[ridxs[3]], roof[ridxs[2]], dthin,
                      jlbrot3, jrfrot3)

        # horizontals across rooflines (and diagonals)
        for diam, pts in [(dthin,roof) , (dthick, rbase)]:
            self.add_pipe(pts[0], pts[3], diam, jlrot2, jrrot2)
            self.add_pipe(pts[4], pts[7], diam, jlrot2, jrrot2)

        # cross-diagonals between front-top and back-top
        self.add_pipe(rbase[2], roof[0], dthin, AUTO, jlfrot3)
        self.add_pipe(rbase[5], roof[7], dthin, AUTO, jrfrot3)

        # finally two uprights from roof, at rear door
        self.add_pipe(rbase[7], midfront[0], dthick, jdown)
        self.add_pipe(rbase[0], midfront[3], dthick, jdown)

        return bbox, self

